@@ -71,8 +71,8 @@ static int remove_dir = 0;
 
 static struct {
     NvCfgBool (*get_pci_devices)(int *, NvCfgPciDevice **);
-    NvCfgBool (*attach_pci_device)(int, int, int, int, NvCfgDeviceHandle *);
-    NvCfgBool (*detach_device)(NvCfgDeviceHandle);
+    NvCfgBool (*open_pci_device)(int, int, int, int, NvCfgDeviceHandle *);
+    NvCfgBool (*close_device)(NvCfgDeviceHandle);
 } nv_cfg_api;
 
 /*
@@ -267,7 +267,7 @@ static NvPdStatus wait_for_init_complete(int pipe_read_fd)
 /*
  * set_device_mode() - This function performs the heavy lifting in enabling or
  * disabling device mode for a given device by performing mode checks and
- * calling libnvidia-cfg to attach and detach the device.
+ * calling libnvidia-cfg to open and close the device.
  */
 static NvPdStatus set_device_mode(NvPdDevice *device, NvPersistenceMode mode)
 {
@@ -284,10 +284,11 @@ static NvPdStatus set_device_mode(NvPdDevice *device, NvPersistenceMode mode)
     switch (mode) {
 
     case NV_PERSISTENCE_MODE_DISABLED:
-        /* If the new mode is disabled, we must detach the device. */
-        success = nv_cfg_api.detach_device(device->nv_cfg_handle);
+
+        /* If the new mode is disabled, we must close the device. */
+        success = nv_cfg_api.close_device(device->nv_cfg_handle);
         if (!success) {
-            syslog_device(&device->pci_info, LOG_ERR, "failed to detach.");
+            syslog_device(&device->pci_info, LOG_ERR, "failed to close.");
             status = NVPD_ERR_DRIVER;
         } else {
             device->nv_cfg_handle = NULL;
@@ -297,14 +298,14 @@ static NvPdStatus set_device_mode(NvPdDevice *device, NvPersistenceMode mode)
 
     case NV_PERSISTENCE_MODE_ENABLED:
 
-        /* If the new mode is enabled, we must attach the device. */
-        success = nv_cfg_api.attach_pci_device(device->pci_info.domain,
-                                               device->pci_info.bus,
-                                               device->pci_info.slot,
-                                               device->pci_info.function,
-                                               &device->nv_cfg_handle);
+        /* If the new mode is enabled, we must open the device. */
+        success = nv_cfg_api.open_pci_device(device->pci_info.domain,
+                                             device->pci_info.bus,
+                                             device->pci_info.slot,
+                                             device->pci_info.function,
+                                             &device->nv_cfg_handle);
         if (!success) {
-            syslog_device(&device->pci_info, LOG_ERR, "failed to attach.");
+            syslog_device(&device->pci_info, LOG_ERR, "failed to open.");
             status = NVPD_ERR_DRIVER;
         }
 
@@ -539,10 +540,10 @@ static NvPdStatus setup_nvidia_cfg_api(const char *nvidia_cfg_path)
     /* Attempt to load all symbols required. */
     status |= load_nvidia_cfg_sym((void **)&nv_cfg_api.get_pci_devices,
                                   "nvCfgGetPciDevices");
-    status |= load_nvidia_cfg_sym((void **)&nv_cfg_api.attach_pci_device,
-                                  "nvCfgAttachPciDevice");
-    status |= load_nvidia_cfg_sym((void **)&nv_cfg_api.detach_device,
-                                  "nvCfgDetachDevice");
+    status |= load_nvidia_cfg_sym((void **)&nv_cfg_api.open_pci_device,
+                                  "nvCfgOpenPciDevice");
+    status |= load_nvidia_cfg_sym((void **)&nv_cfg_api.close_device,
+                                  "nvCfgCloseDevice");
 
     if (status != 0) {
         /* Missing symbols are already called out by load_nvidia_cfg_sym(). */
